/**********************************************************************
 * Copyright (c) 2012, 2014 Ericsson
 *
 * All rights reserved. This program and the accompanying materials are
 * made available under the terms of the Eclipse Public License v1.0 which
 * accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *   Bernd Hufmann - Initial API and implementation
 *   Markus Schorn - Bug 448058: Use org.eclipse.remote in favor of RSE
 **********************************************************************/
package org.eclipse.linuxtools.internal.lttng2.control.ui.views.remote;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.remote.core.IRemoteConnectionChangeListener;
import org.eclipse.remote.core.IRemoteFileManager;
import org.eclipse.remote.core.IRemoteProcessBuilder;

/**
 * <p>
 * Remote System Proxy interface.
 * </p>
 *
 * @author Bernd Hufmann
 */
public interface IRemoteSystemProxy {

    // ------------------------------------------------------------------------
    // Constants
    // ------------------------------------------------------------------------
    /**
     * Invalid port number for IP based connections.
     */
    static final int INVALID_PORT_NUMBER = -1;

    // ------------------------------------------------------------------------
    // Operations
    // ------------------------------------------------------------------------

    /**
     * Finds the File Service Subsystem.
     *
     * @return file service subsystem, or <code>null</code> if not found.
     */
    IRemoteFileManager getFileServiceSubSystem();

    /**
     * Returns a remote process builder for remote launching a process.
     * @param command the command to be executed.
     * @return the builder, or <code>null</code> if not possible.
     */
    IRemoteProcessBuilder getProcessBuilder(String...command);


    /**
     * Connects the shell service sub system.
     * @param monitor a monitor to report progress.
     *
     * @param callback
     *            - call-back method being called when connection was finished
     * @throws ExecutionException
     *             If the connection fails
     */
    void connect(IProgressMonitor monitor) throws ExecutionException;

    /**
     * Disconnects from the shell service sub system.
     *
     * @throws ExecutionException
     *             If the disconnect command fails
     */
    void disconnect() throws ExecutionException;

    /**
     * Disposes the proxy, may close the connection.
     */
    void dispose();

    /**
     * Creates a command shell.
     *
     * @return the command shell implementation
     * @throws ExecutionException
     *             If the command fails
     */
    ICommandShell createCommandShell() throws ExecutionException;

    /**
     * Method to add a communication listener to the connector service defined
     * for the given connection.
     *
     * @param listener
     *            - listener to add
     */
    void addConnectionChangeListener(IRemoteConnectionChangeListener listener);

    /**
     * Method to remove a communication listener from the connector service
     * defined for the given connection.
     *
     * @param listener
     *            - listener to remove
     */
    void removeConnectionChangeListener(IRemoteConnectionChangeListener listener);

    /**
     * @return whether the remote host is currently connected.
     */
    boolean isConnected();
}